@@ -20,6 +20,7 @@ import copy
 import logging
 import os
 import platform
+import warnings
 
 from botocore import __version__
 import botocore.configloader
@@ -27,8 +28,10 @@ import botocore.credentials
 import botocore.client
 from botocore.exceptions import ConfigNotFound, ProfileNotFound
 from botocore.exceptions import UnknownServiceError, PartialCredentialsError
+from botocore.errorfactory import ClientExceptionsFactory
 from botocore import handlers
 from botocore.hooks import HierarchicalEmitter, first_non_none_response
+from botocore.hooks import EventAliaser
 from botocore.loaders import create_loader
 from botocore.parsers import ResponseParserFactory
 from botocore.regions import EndpointResolver
@@ -36,6 +39,10 @@ from botocore.model import ServiceModel
 from botocore import paginate
 from botocore import waiter
 from botocore import retryhandler, translate
+from botocore.utils import EVENT_ALIASES
+
+
+logger = logging.getLogger(__name__)
 
 
 class Session(object):
@@ -134,9 +141,10 @@ class Session(object):
         if session_vars:
             self.session_var_map.update(session_vars)
         if event_hooks is None:
-            self._events = HierarchicalEmitter()
+            self._original_handler = HierarchicalEmitter()
         else:
-            self._events = event_hooks
+            self._original_handler = event_hooks
+        self._events = EventAliaser(self._original_handler)
         if include_builtin_handlers:
             self._register_builtin_handlers(self._events)
         self.user_agent_name = 'Botocore'
@@ -156,6 +164,7 @@ class Session(object):
             self._session_instance_vars['profile'] = profile
         self._client_config = None
         self._components = ComponentLocator()
+        self._internal_components = ComponentLocator()
         self._register_components()
 
     def _register_components(self):
@@ -164,6 +173,7 @@ class Session(object):
         self._register_endpoint_resolver()
         self._register_event_emitter()
         self._register_response_parser_factory()
+        self._register_exceptions_factory()
 
     def _register_event_emitter(self):
         self._components.register_component('event_emitter', self._events)
@@ -183,13 +193,17 @@ class Session(object):
             loader = self.get_component('data_loader')
             endpoints = loader.load_data('endpoints')
             return EndpointResolver(endpoints)
-        self._components.lazy_register_component(
+        self._internal_components.lazy_register_component(
             'endpoint_resolver', create_default_resolver)
 
     def _register_response_parser_factory(self):
         self._components.register_component('response_parser_factory',
                                             ResponseParserFactory())
 
+    def _register_exceptions_factory(self):
+        self._internal_components.register_component(
+            'exceptions_factory', ClientExceptionsFactory())
+
     def _register_builtin_handlers(self, events):
         for spec in handlers.BUILTIN_HANDLERS:
             if len(spec) == 2:
@@ -253,12 +267,29 @@ class Session(object):
         value = None
         var_config = self.session_var_map[logical_name]
         if self._found_in_instance_vars(methods, logical_name):
-            return self._session_instance_vars[logical_name]
+            value = self._session_instance_vars[logical_name]
+            logger.debug(
+                "Loading variable %s from instance vars with value %r.",
+                logical_name,
+                value,
+            )
+            return value
         elif self._found_in_env(methods, var_config):
             value = self._retrieve_from_env(var_config[1], os.environ)
+            logger.debug(
+                "Loading variable %s from environment with value %r.",
+                logical_name,
+                value,
+            )
         elif self._found_in_config_file(methods, var_config):
             value = self.get_scoped_config()[var_config[0]]
+            logger.debug(
+                "Loading variable %s from config file with value %r.",
+                logical_name,
+                value,
+            )
         if value is None:
+            logger.debug("Loading variable %s from defaults.", logical_name)
             value = var_config[2]
         if var_config[3] is not None:
             value = var_config[3](value)
@@ -314,6 +345,11 @@ class Session(object):
         :param value: The value to associate with the config variable.
 
         """
+        logger.debug(
+            "Setting config variable for %s to %r",
+            logical_name,
+            value,
+        )
         self._session_instance_vars[logical_name] = value
 
     def get_scoped_config(self):
@@ -526,7 +562,8 @@ class Session(object):
             type_name='service-2',
             api_version=api_version
         )
-        self._events.emit('service-data-loaded.%s' % service_name,
+        service_id = EVENT_ALIASES.get(service_name, service_name)
+        self._events.emit('service-data-loaded.%s' % service_id,
                           service_data=service_data,
                           service_name=service_name, session=self)
         return service_data
@@ -692,7 +729,29 @@ class Session(object):
         return first_non_none_response(responses)
 
     def get_component(self, name):
-        return self._components.get_component(name)
+        try:
+            return self._components.get_component(name)
+        except ValueError:
+            if name in ['endpoint_resolver', 'exceptions_factory']:
+                warnings.warn(
+                    'Fetching the %s component with the get_component() '
+                    'method is deprecated as the component has always been '
+                    'considered an internal interface of botocore' % name,
+                    DeprecationWarning)
+                return self._internal_components.get_component(name)
+            raise
+
+    def _get_internal_component(self, name):
+        # While this method may be called by botocore classes outside of the
+        # Session, this method should **never** be used by a class that lives
+        # outside of botocore.
+        return self._internal_components.get_component(name)
+
+    def _register_internal_component(self, name, component):
+        # While this method may be called by botocore classes outside of the
+        # Session, this method should **never** be used by a class that lives
+        # outside of botocore.
+        return self._internal_components.register_component(name, component)
 
     def register_component(self, name, component):
         self._components.register_component(name, component)
@@ -817,10 +876,12 @@ class Session(object):
                                                  aws_secret_access_key))
         else:
             credentials = self.get_credentials()
-        endpoint_resolver = self.get_component('endpoint_resolver')
+        endpoint_resolver = self._get_internal_component('endpoint_resolver')
+        exceptions_factory = self._get_internal_component('exceptions_factory')
         client_creator = botocore.client.ClientCreator(
             loader, endpoint_resolver, self.user_agent(), event_emitter,
-            retryhandler, translate, response_parser_factory)
+            retryhandler, translate, response_parser_factory,
+            exceptions_factory)
         client = client_creator.create_client(
             service_name=service_name, region_name=region_name,
             is_secure=use_ssl, endpoint_url=endpoint_url, verify=verify,
@@ -841,7 +902,7 @@ class Session(object):
         :rtype: list
         :return: Returns a list of partition names (e.g., ["aws", "aws-cn"])
         """
-        resolver = self.get_component('endpoint_resolver')
+        resolver = self._get_internal_component('endpoint_resolver')
         return resolver.get_available_partitions()
 
     def get_available_regions(self, service_name, partition_name='aws',
@@ -864,7 +925,7 @@ class Session(object):
              fips-us-gov-west-1, etc).
         :return: Returns a list of endpoint names (e.g., ["us-east-1"]).
         """
-        resolver = self.get_component('endpoint_resolver')
+        resolver = self._get_internal_component('endpoint_resolver')
         results = []
         try:
             service_data = self.get_service_data(service_name)
